import { Component, EventEmitter, Output, Input } from '@angular/core';
import { DataService } from 'src/services/data/data.service';
import { CdkDragDrop, moveItemInArray } from '@angular/cdk/drag-drop';
import { Spell } from 'src/interfaces/spell';

@Component({
  selector: 'spell-table',
  templateUrl: './spell-table.component.html',
  styleUrls: ['./spell-table.component.scss'],
})
export class SpellTableComponent {
  public constructor(public dataAccessor: DataService) {}

  @Output() public createNewSpell: EventEmitter<void> =
    new EventEmitter<void>();
  @Output() public updateSpellEvent: EventEmitter<any> =
    new EventEmitter<any>();

  @Input() public spellAttackBonus: string = '0';
  @Input() public spellSaveDC: number = 0;

  public spells!: Spell[];

  public attributes: any = {
    strength: 'STR',
    dexterity: 'DEX',
    constitution: 'CON',
    intelligence: 'INT',
    wisdom: 'WIS',
    charisma: 'CHA',
  };

  public ngOnInit(): void {
    this.spells = this.dataAccessor.getSpells();
  }

  public dropSpells(event: CdkDragDrop<string[]>): void {
    moveItemInArray(this.spells, event.previousIndex, event.currentIndex);
    this.updateSpellsInDatabase();
  }

  public updateSpellsInDatabase(): void {
    this.dataAccessor.setSpells(this.spells);
  }

  public createSpell(): void {
    console.log('createWeapon()');
    this.createNewSpell.emit();
  }

  public addSpell(spell: Spell) {
    this.spells.push(spell);
    this.updateSpellsInDatabase();
  }

  public emitUpdateSpell(index: number): void {
    this.updateSpellEvent.emit({ spell: this.spells[index], index: index });
    console.log('emitUpdateWeapon()');
  }

  public updateSpell(spell: Spell, index: number): void {
    this.spells[index] = spell;
    this.updateSpellsInDatabase();
  }

  public deleteSpell(index: number): void {
    this.spells.splice(index, 1);
    this.updateSpellsInDatabase();
  }
}
