import { Component, ViewChild } from '@angular/core';
import { Weapon } from 'src/interfaces/weapon';
import { Spell } from 'src/interfaces/spell';
import { NgxSmartModalService } from 'ngx-smart-modal';
import { WeaponTableComponent } from './weapon-table/weapon-table.component';
import { SpellTableComponent } from './spell-table/spell-table.component';
import { DataService } from 'src/services/data/data.service';

@Component({
  selector: 'app-weapons-container',
  templateUrl: './weapons-container.component.html',
  styleUrls: ['./weapons-container.component.scss'],
})
export class WeaponsContainerComponent {
  public constructor(
    public ngxSmartModalService: NgxSmartModalService,
    private dataAccessor: DataService
  ) {}

  @ViewChild(WeaponTableComponent) weaponTable!: WeaponTableComponent;
  @ViewChild(SpellTableComponent) spellTable!: SpellTableComponent;

  public active: number = 1;

  public spellAttackBonus: string = '0';
  public spellSaveDC: number = 0;
  private spellcastingAttribute: string | undefined;
  private spellcastingAttributeModifier: number = 0;
  private proficiencyBonus: number = 2;

  public ngOnInit(): void {
    //this.dataAccessor.getSpellcastingAttribute() oder so
    this.spellcastingAttribute = 'intelligence';

    this.dataAccessor.proficiency$.subscribe((value) => {
      this.proficiencyBonus = value.value;
      if (this.spellcastingAttribute) {
        this.computeSpellAttackBonusAndSaveDC();
      }
    });

    this.dataAccessor.intelligence$.subscribe((value) => {
      this.spellcastingAttributeModifier = Math.floor((value.value - 10) / 2);
      this.computeSpellAttackBonusAndSaveDC();
    });
  }

  // spells
  public openSpellCreationModal(): void {
    this.ngxSmartModalService.getModal('spellModal').open();
    console.log('openSpellModal()');
  }

  public addNewlyCreatedSpell(spell: Spell): void {
    this.spellTable.addSpell(spell);
  }

  public openSpellUpdateModal(data: any): void {
    this.ngxSmartModalService.getModal('spellModal').setData(data);
    this.ngxSmartModalService.getModal('spellModal').open();
  }

  public updateSpellInTable(data: any): void {
    console.log('updateSpellInTable()', data);
    this.spellTable.updateSpell(data.spell, data.index);
  }

  public deleteSpellInTable(index: number): void {
    console.log('deleteSpellInTable()', index);
    this.spellTable.deleteSpell(index);
  }

  private computeSpellAttackBonusAndSaveDC(): void {
    let attackBonus =
      this.spellcastingAttributeModifier + this.proficiencyBonus;
    this.spellSaveDC = 8 + attackBonus;
    if (attackBonus >= 0) {
      this.spellAttackBonus = '+' + attackBonus;
    } else {
      this.spellAttackBonus = attackBonus.toString();
    }
  }

  // weapons
  public openWeaponCreationModal(): void {
    console.log('openWeaponModal()');
    this.ngxSmartModalService.getModal('weaponModal').open();
  }

  public addNewlyCreatedWeapon(weapon: Weapon): void {
    this.weaponTable.addWeapon(weapon);
  }

  public openWeaponUpdateModal(data: any): void {
    this.ngxSmartModalService.getModal('weaponModal').setData(data);
    this.ngxSmartModalService.getModal('weaponModal').open();
  }

  public updateWeaponInTable(data: any): void {
    this.weaponTable.updateWeapon(data.weapon, data.index);
  }

  public deleteWeaponInTable(index: number): void {
    this.weaponTable.deleteWeapon(index);
  }
}
